/**
 * Copyright 2008 Robert Ostronic <a href="mailto:rostronic@gmail.com">rostronic@gmail.com</a>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.digitalsanctuary.atg.captcha;

import java.io.IOException;

import javax.servlet.ServletException;

import atg.core.util.StringUtils;
import atg.droplet.DropletFormException;
import atg.droplet.GenericFormHandler;
import atg.servlet.DynamoHttpServletRequest;
import atg.servlet.DynamoHttpServletResponse;
import atg.userprofiling.Profile;

/**
 * Handles validation of user input against captcha input. Depends on simplecaptcha image generation. If simplecaptcha
 * value is not found in session user passes validation.<br />
 * <br />
 * Handlers:<br />
 * {@link #handleSubmit handleSubmit}<br />
 * 
 * @author Robert Ostronic <a href="mailto:rostronic@gmail.com">rostronic@gmail.com</a>
 * @author Devon Hillard (devon@digitalsanctuary.com)
 * 
 */
public class CaptchaFormHandler extends GenericFormHandler {

    /**
     * The captcha validated profile property.
     */
    public static final String PROFILE_CAPTCHA_VALIDATED = "captchaValidated";

    /**
     * Session parameter name where captcha text is placed in session by SimpleCaptcha.
     */
    public static final String SESSION_CAPTCHA_PROPERTY = "SIMPLE_CAPCHA_SESSION_KEY";

    /**
     * The URL to redirect the user to if the captcha was entered correctly.
     */
    private String mSuccessURL;

    /**
     * The URL to redirect the user to if the captcha was entered incorrectly.
     */
    private String mErrorURL;

    /**
     * The user input captcha value.
     */
    private String mCaptcha;

    /**
     * The user's Profile component.
     */
    private Profile mProfile;

    /**
     * This method handles the submission of a captcha answer.
     * 
     * @param pRequest
     *                the dynamo request object.
     * @param pResponse
     *                the dynamo response object.
     * @return true if page should render normally.
     * @throws IOException
     *                 on error.
     * @throws ServletException
     *                 on error.
     */
    public boolean handleSubmit(final DynamoHttpServletRequest pRequest, final DynamoHttpServletResponse pResponse)
	    throws IOException, ServletException {
	// check if already validated in profile or already registered
	if ((this.getProfile() != null) && !this.getProfile().isTransient()) {
	    return checkFormRedirect(getSuccessURL(), getErrorURL(), pRequest, pResponse);
	} else if ((this.getProfile() != null)
		&& ((Boolean) this.getProfile().getPropertyValue(PROFILE_CAPTCHA_VALIDATED)).booleanValue()) {
	    return checkFormRedirect(getSuccessURL(), getErrorURL(), pRequest, pResponse);
	}

	// validate against value in session
	final String sCaptchaValue = (String) pRequest.getSession().getAttribute(SESSION_CAPTCHA_PROPERTY);
	if (sCaptchaValue == null) {
	    addFormException(new DropletFormException("Your browser must allow cookies.", "captcha", "captcha"));
	    // add param to returnURL for error message display
	    setErrorURL(getErrorURL() + "&noCookie=true");
	} else if (StringUtils.isBlank(this.getCaptcha())) {
	    addFormException(new DropletFormException("You must supply a value for captcha text.", "captcha", "captcha"));
	} else if ((sCaptchaValue != null) && !(sCaptchaValue).equals(this.getCaptcha())) {
	    addFormException(new DropletFormException("The value provided for captcha text was incorrect.", "captcha",
		    "captcha"));
	} else {
	    // captcha validated
	    postValidationAction();
	}
	return checkFormRedirect(getSuccessURL(), getErrorURL(), pRequest, pResponse);
    }

    /**
     * When this class is sub-typed, this method should be overridden to perform any actions upon the user entering the
     * correct captcha. For instance setting a profile attribute or a property on a session scoped component.
     */
    public void postValidationAction() {

    }

    /**
     * @return the successURL.
     */
    public String getSuccessURL() {
	return mSuccessURL;
    }

    /**
     * @param pSuccessURL
     *                the url to redirect to in a success case.
     */
    public void setSuccessURL(final String pSuccessURL) {
	mSuccessURL = pSuccessURL;
    }

    /**
     * @return the error URL.
     */
    public String getErrorURL() {
	return mErrorURL;
    }

    /**
     * @param pErrorURL
     *                the url to redirect to in an error case.
     */
    public void setErrorURL(final String pErrorURL) {
	mErrorURL = pErrorURL;
    }

    /**
     * @return the captcha
     */
    public String getCaptcha() {
	return mCaptcha;
    }

    /**
     * @param pCaptcha
     *                the captcha to set
     */
    public void setCaptcha(final String pCaptcha) {
	mCaptcha = pCaptcha;
    }

    /**
     * @return the profile
     */
    public Profile getProfile() {
	return mProfile;
    }

    /**
     * @param pProfile
     *                the profile to set
     */
    public void setProfile(final Profile pProfile) {
	mProfile = pProfile;
    }
}
