/**
 * Copyright 2008 Devon Hillard (devon@digitalsanctuary.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */
package com.digitalsanctuary.atg.crypto;

import atg.nucleus.GenericService;
import atg.nucleus.ServiceException;

/**
 * This is an abstract class of an encryptor component.
 * 
 * @author Louis Baliotis
 * @created Aug 24, 2006 9:17:43 AM
 */
public abstract class AbstractEncryptorComponent extends GenericService {

    /**
     * This is the local AbstractEncryptor instance which this component will use to perform the encryption and
     * decryption operations.
     */
    private AbstractEncryptor mEncryptor;

    /**
     * This is the Key which will be used to instantiate the AESEncryptor.
     */
    private String mKey;

    /**
     * This method computes the MAC message signature for the clear text. This is basically a checksum so you know you
     * decrypted the data successfully and that it wasn't tampered with.
     * 
     * @param pClearText
     *                the plain text you wish to compute the MAC for. This must match the plain text you encrypt.
     * @return a String, which is a Base64 encoded version of the checksum signature of the cleartext, based on the
     *         current algorithm and key.
     * @throws EncryptorException
     *                 if the key is invalid, if the algorithm is improperly defined, or if the encoding fails.
     */
    public String computeMAC(final String pClearText) throws EncryptorException {
	if (isLoggingDebug()) {
	    logDebug("AbstractEncryptorComponent.computeMAC() - Entering...");
	}
	final String hmacString = this.mEncryptor.computeMAC(pClearText);
	if (isLoggingDebug()) {
	    logDebug("AbstractEncryptorComponent.computeMAC() - Returning HMAC: " + hmacString);
	}
	return hmacString;
    }

    /**
     * This method takes in a byte array and decrypts it using the AES algorithm and the key used to setup this class.
     * 
     * @param pBytesToDecrypt
     *                the byte array to be decrypted using AES and the key in mKeyBytes.
     * @return a byte array of decrypted data.
     * @throws EncryptorException
     *                 if decryption fails.
     */
    public byte[] decryptBytes(final byte[] pBytesToDecrypt) throws EncryptorException {
	return this.mEncryptor.decrypt(pBytesToDecrypt);
    }

    /**
     * This method takes in a String of a Base64 encoded byte array of encrypted data and decrypts it using the AES
     * algorithm and the key used to setup this class.
     * 
     * @param pEncryptedContent
     *                the String of encoded and encrypted content which should be decrypted.
     * @return a byte array of decrypted data.
     * @throws EncryptorException
     *                 if decryption fails.
     */
    public String decryptString(final String pEncryptedContent) throws EncryptorException {
	return this.mEncryptor.decrypt(pEncryptedContent);
    }

    /**
     * This method sets up our component, and creates the Encryptor object.
     * 
     * @see atg.nucleus.GenericService#doStartService()
     * @throws ServiceException
     *                 if key is null
     */
    public void doStartService() throws ServiceException {
	super.doStartService();

	// Check our properties.
	if (getKey() == null) {
	    throw new ServiceException("AbstractEncryptorComponent.key has not been set properly. Please check your "
		    + "environment configuration and try restarting.");
	}

	// Set up our Encryptor.
	try {
	    setupEncryptor();
	} catch (EncryptorException ee) {
	    throw new ServiceException(
		    "AbstractEncryptorComponent could not create an AESEncryptor object using the given "
			    + "\"key\" property. Please check your environment configuration and try restarting.");
	}
	if (this.mEncryptor == null) {
	    throw new ServiceException("AbstractEncryptorComponent created a NULL AESEncryptor object using the given "
		    + "\"key\" property. Please check your environment configuration and try restarting.");
	}
    }

    /**
     * This method sets up the encryptor. It must be overriden by the actual type of encryptor component you wish to
     * use. An example body would look like this:
     * 
     * final byte[] keyBytes = Base64.decodeToByteArray(getKey()); this.mEncryptor = new AbstractEncryptor(keyBytes);
     * 
     * @throws EncryptorException
     *                 if the encryptor could not be created
     */
    public abstract void setupEncryptor() throws EncryptorException;

    /**
     * This method takes in a byte array and encrypts it using the AES algorithm and the key used to setup this class.
     * 
     * @param pBytesToEncrypt
     *                the byte array to be encrypted using AES and the key in mKeyBytes
     * @return a byte array of encrypted data
     * @throws EncryptorException
     *                 if encryption fails.
     */
    public byte[] encryptBytes(final byte[] pBytesToEncrypt) throws EncryptorException {
	return this.mEncryptor.encrypt(pBytesToEncrypt);
    }

    /**
     * This method takes in a String and encrypts is using the AES algorithm and the key used to setup this class.
     * 
     * @param pUnencryptedContent
     *                the String to be encrypted using AES and the key in mKey.
     * @return a String of Base64 encoded, encrypted bytes.
     * @throws EncryptorException
     *                 if encryption fails.
     */
    public String encryptString(final String pUnencryptedContent) throws EncryptorException {
	return this.mEncryptor.encrypt(pUnencryptedContent);
    }

    /**
     * @return Returns the key.
     */
    public String getKey() {
	return this.mKey;
    }

    /**
     * @param pKey
     *                The key to set.
     */
    public void setKey(final String pKey) {
	this.mKey = pKey;
    }

    /**
     * @return the encryptor
     */
    public AbstractEncryptor getEncryptor() {
	return this.mEncryptor;
    }

    /**
     * @param pEncryptor
     *                the encryptor to set
     */
    public void setEncryptor(final AbstractEncryptor pEncryptor) {
	this.mEncryptor = pEncryptor;
    }
}
